.IFNDEF UNIFIEDMUL
.EQU UNIFIEDMUL=1

;If your target device doesn't own a hardware multiplier and therefore
;not provides the mul instruction, the UMUL macro provides the functionality
;of this instruction implemented using the available instructions.
;The macro provides two drawbacks too: It takes 40 instead of only two clocks
;and it needs a register anytime containing a value of zero. This cannot be
;R0 nor R1. You must provide a NULL register using the .DEF directive to make
;the macro work.

;->@0   : factor 1 register
;  @1   : factor 2 register
;  @2   : NULL register
;  CF   : cleared
;  R1:R0: partial sum
;  @3   : binary digit to calculate
;<-R1:R0: new partial sum
;  CF   : cleared
.MACRO UMUL_ADDDIGIT
	sbrc @0,@3	; 2	 1
	add R0,@1	;	 1
	adc R1,@2	; 1	 1
.ENDMACRO		;--	--
				; 3	 3

;->R1:R0: partial sum
;<-R1:R0: left shifted partial sum
.MACRO UMUL_LSLDIGIT
	lsl R0		; 1
	rol R1		; 1
.ENDMACRO		;--
				; 2

;->@0: factor 1 register
;  @1: factor 2 register
;  @2: NULL register
.MACRO UMUL_CORE
	sub R0,R0					; 1
	sub R1,R1					; 1
	UMUL_ADDDIGIT @0, @1, @2, 7	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 6	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 5	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 4	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 3	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 2	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 1	; 3
	UMUL_LSLDIGIT				; 2
	UMUL_ADDDIGIT @0, @1, @2, 0	; 3
.ENDMACRO						;--
								;40

;->@0   : factor 1 register
;  @1   : factor 2 register
;<-R1:R0: result of multiplication of the two register contents
.MACRO UMUL
.IFDEF RWW_STOP_ADDR
	.IF RWW_STOP_ADDR
		mul @0,@1 
	.ELSE
		UMUL_CORE @0,@1,NULL
		.WARNING "38 extra ticks (software mul)"
	.ENDIF	
.ELSE
	UMUL_CORE @0,@1,NULL
	.WARNING "38 extra ticks (software mul)"
.ENDIF
.ENDMACRO

.ENDIF
